@@ -1,3 +1,40 @@
+"""
+
+######### Version 1
+
+# Ends up executing all the questions' bodies
+with Pool("P1") as pool:
+    with Question("QA", "Create a for loop"):
+        ensure_ast("For")
+    with Question("QB", "Create an if statement"):
+        ensure_ast("If")
+pool.choose().ask()
+
+######### Version 2
+
+pool = Pool("P1", [question_A, question_B])
+
+@pool.Question("Q", "Create a for loop")
+def question_A():
+    ensure_ast("For")
+
+@pool.Question("QB", "Create an if statement")
+def question_B():
+    ensure_ast("If")
+
+pool.choose().ask()
+
+######### Version 3
+
+with Pool("P1") as pool:
+    if pool.asked_question("QA", "Create a for loop"):
+        ensure_ast("For")
+    if pool.asked_question("QB", "Create an if statement"):
+        ensure_ast("If")
+
+"""
+
+
 from pedal.core.feedback import Feedback
 from pedal.core.feedback_category import FeedbackCategory, FeedbackKind
 from pedal.core.report import MAIN_REPORT
